@@ -32,28 +32,30 @@ module Criterion.Report
     ) where
 
 import Control.Exception (Exception, IOException, throwIO)
-import Control.Monad (mplus, unless)
+import Control.Monad (mplus)
 import Control.Monad.IO.Class (MonadIO(liftIO))
 import Control.Monad.Reader (ask)
 import Criterion.Monad (Criterion)
 import Criterion.Types
-import Data.Aeson (ToJSON (..), Value(..), object, (.=), Value, encode)
+import Data.Aeson (ToJSON (..), Value(..), object, (.=), Value)
+import Data.Aeson.Text (encodeToLazyText)
 import Data.Data (Data, Typeable)
 import Data.Foldable (forM_)
 import GHC.Generics (Generic)
 import Paths_criterion (getDataFileName)
 import Statistics.Function (minMax)
-import Statistics.Types (confidenceInterval, confidenceLevel, confIntCL, estError)
 import System.Directory (doesFileExist)
 import System.FilePath ((</>), (<.>), isPathSeparator)
 import System.IO (hPutStrLn, stderr)
-import Text.Microstache (Key (..), MustacheWarning (..), Node (..), Template (..),
-                         compileMustacheText, displayMustacheWarning, renderMustacheW)
+import Text.Microstache (Key (..), Node (..), Template (..),
+                compileMustacheText, displayMustacheWarning, renderMustacheW)
 import Prelude ()
 import Prelude.Compat
 import qualified Control.Exception as E
 import qualified Data.Text as T
+#if defined(EMBED)
 import qualified Data.Text.Lazy.Encoding as TLE
+#endif
 import qualified Data.Text.IO as T
 import qualified Data.Text.Lazy as TL
 import qualified Data.Text.Lazy.IO as TL
@@ -61,13 +63,15 @@ import qualified Data.Vector.Generic as G
 import qualified Data.Vector.Unboxed as U
 
 #if defined(EMBED)
-import Criterion.EmbeddedData (dataFiles, jQueryContents, flotContents,
-                               flotErrorbarsContents, flotNavigateContents)
+import Criterion.EmbeddedData (dataFiles, chartContents)
 import qualified Data.ByteString.Lazy as BL
 import qualified Data.Text.Encoding as TE
 #else
-import qualified Language.Javascript.Flot as Flot
-import qualified Language.Javascript.JQuery as JQuery
+import qualified Language.Javascript.Chart as Chart
+#endif
+
+#if MIN_VERSION_aeson(2,0,0)
+import qualified Data.Aeson.Key as Key
 #endif
 
 -- | Trim long flat tails from a KDE plot.
@@ -104,6 +108,41 @@ report reports = do
     tpl <- loadTemplate [td,"."] template
     TL.writeFile name =<< formatReport reports tpl
 
+-- | Escape JSON string aimed to be embedded in an HTML <script> tag.  Notably
+-- < and > are replaced with their unicode escape sequences such that closing
+-- the <script> tag from within the JSON data is disallowed, i.e, the character
+-- sequence "</" is made impossible.
+--
+-- Moreover, & is escaped to avoid HTML character references (&<code>;), + is
+-- escaped to avoid UTF-7 attacks (should only affect old versions of IE), and
+-- \0 is escaped to allow it to be represented in JSON, as the NUL character is
+-- disallowed in JSON but valid in Haskell characters.
+--
+-- The following characters are replaced with their unicode escape sequences
+-- (\uXXXX):
+-- <, >, &, +, \x2028 (line separator), \x2029 (paragraph separator), and \0
+-- (null terminator)
+--
+-- Other characters are such as \\ (backslash) and \n (newline) are not escaped
+-- as the JSON serializer @encodeToLazyText@ already escapes them when they
+-- occur inside JSON strings and they cause no issues with respect to HTML
+-- safety when used outside of strings in the JSON-encoded payload.
+--
+-- If the resulting JSON-encoded Text is embedded in an HTML attribute, extra
+-- care is required to also escape quotes with character references in the
+-- final JSON payload.
+-- See <https://html.spec.whatwg.org/multipage/syntax.html#syntax-attributes>
+-- for details on how to escape attribute values.
+escapeJSON :: Char -> TL.Text
+escapeJSON '<'      = "\\u003c" -- ban closing of the script tag by making </ impossible
+escapeJSON '>'      = "\\u003e" -- encode tags with unicode escape sequences
+escapeJSON '\x2028' = "\\u2028" -- line separator
+escapeJSON '\x2029' = "\\u2029" -- paragraph separator
+escapeJSON '&'      = "\\u0026" -- avoid HTML entities
+escapeJSON '+'      = "\\u002b" -- + can be used in UTF-7 escape sequences
+escapeJSON '\0'     = "\\u0000" -- make null characters explicit
+escapeJSON c        = TL.singleton c
+
 -- | Format a series of 'Report' values using the given Mustache template.
 formatReport :: [Report]
              -> TL.Text    -- ^ Mustache template.
@@ -113,25 +152,18 @@ formatReport reports templateName = do
         Left err -> fail (show err) -- TODO: throw a template exception?
         Right x -> return x
 
-    jQuery            <- jQueryFileContents
-    flot              <- flotFileContents
-    flotErrorbars     <- flotErrorbarsFileContents
-    flotNavigate      <- flotNavigateFileContents
-    jQueryCriterionJS <- readDataFile ("js" </> "jquery.criterion.js")
-    criterionCSS      <- readDataFile "criterion.css"
+    criterionJS <- readDataFile "criterion.js"
+    criterionCSS <- readDataFile "criterion.css"
+    chartJS <- chartFileContents
 
     -- includes, only top level
     templates <- getTemplateDir
     template <- includeTemplate (includeFile [templates]) template0
 
     let context = object
-            [ "json"                .= reports
-            , "report"              .= map inner reports
-            , "js-jquery"           .= jQuery
-            , "js-flot"             .= flot
-            , "js-flot-errorbars"   .= flotErrorbars
-            , "js-flot-navigate"    .= flotNavigate
-            , "jquery-criterion-js" .= jQueryCriterionJS
+            [ "json"                .= reportsJSON reports
+            , "js-criterion"        .= criterionJS
+            , "js-chart"            .= chartJS
             , "criterion-css"       .= criterionCSS
             ]
 
@@ -139,33 +171,17 @@ formatReport reports templateName = do
     -- If there were any issues during mustache template rendering, make sure
     -- to inform the user. See #127.
     forM_ warnings $ \warning -> do
-      -- The one thing we choose not to warn about is substituting in the `json`
-      -- key. The reason is that `json` is used in:
-      --
-      --   var reports = {{{json}}};
-      --
-      -- So `json` represents a raw JavaScript array. This is a bit skeevy by
-      -- mustache conventions, but redesigning the template to avoid this
-      -- warning would be more work than just substituting the array directly.
-      unless (warning == MustacheDirectlyRenderedValue (Key ["json"])) $
-        mapM_ (hPutStrLn stderr)
-         [ "criterion: warning:"
-         , "  " ++ displayMustacheWarning warning
-         , ""
-         ]
+        criterionWarning $ displayMustacheWarning warning
     return formatted
   where
-    jQueryFileContents, flotFileContents :: IO T.Text
+    reportsJSON :: [Report] -> T.Text
+    reportsJSON = TL.toStrict . TL.concatMap escapeJSON . encodeToLazyText
+
+    chartFileContents :: IO T.Text
 #if defined(EMBED)
-    jQueryFileContents        = pure $ TE.decodeUtf8 jQueryContents
-    flotFileContents          = pure $ TE.decodeUtf8 flotContents
-    flotErrorbarsFileContents = pure $ TE.decodeUtf8 flotErrorbarsContents
-    flotNavigateFileContents  = pure $ TE.decodeUtf8 flotNavigateContents
+    chartFileContents        = pure $ TE.decodeUtf8 chartContents
 #else
-    jQueryFileContents        = T.readFile =<< JQuery.file
-    flotFileContents          = T.readFile =<< Flot.file Flot.Flot
-    flotErrorbarsFileContents = T.readFile =<< Flot.file Flot.FlotErrorbars
-    flotNavigateFileContents  = T.readFile =<< Flot.file Flot.FlotNavigate
+    chartFileContents        = T.readFile =<< Chart.file Chart.Chart
 #endif
 
     readDataFile :: FilePath -> IO T.Text
@@ -188,43 +204,12 @@ formatReport reports templateName = do
         fmap TextBlock (f (T.unpack fp))
     includeNode _ n = return n
 
-    -- Merge Report with it's analysis and outliers
-    merge :: ToJSON a => a -> Value -> Value
-    merge x y = case toJSON x of
-        Object x' -> case y of
-            Object y' -> Object (x' <> y')
-            _         -> y
-        _         -> y
-
-    inner r@Report {..} = merge reportAnalysis $ merge reportOutliers $ object
-        [ "name"                  .= reportName
-        , "json"                  .= TLE.decodeUtf8 (encode r)
-        , "number"                .= reportNumber
-        , "iters"                 .= vector "x" iters
-        , "times"                 .= vector "x" times
-        , "cycles"                .= vector "x" cycles
-        , "kdetimes"              .= vector "x" kdeValues
-        , "kdepdf"                .= vector "x" kdePDF
-        , "kde"                   .= vector2 "time" "pdf" kdeValues kdePDF
-        , "anMeanConfidenceLevel" .= anMeanConfidenceLevel
-        , "anMeanLowerBound"      .= anMeanLowerBound
-        , "anMeanUpperBound"      .= anMeanUpperBound
-        , "anStdDevLowerBound"    .= anStdDevLowerBound
-        , "anStdDevUpperBound"    .= anStdDevUpperBound
-        ]
-      where
-        [KDE{..}]          = reportKDEs
-        SampleAnalysis{..} = reportAnalysis
-
-        iters  = measure measIters reportMeasured
-        times  = measure measTime reportMeasured
-        cycles = measure measCycles reportMeasured
-        anMeanConfidenceLevel
-               = confidenceLevel $ confIntCL $ estError anMean
-        (anMeanLowerBound, anMeanUpperBound)
-               = confidenceInterval anMean
-        (anStdDevLowerBound, anStdDevUpperBound)
-               = confidenceInterval anStdDev
+criterionWarning :: String -> IO ()
+criterionWarning msg =
+  hPutStrLn stderr $ unlines
+    [ "criterion: warning:"
+    , "  " ++ msg
+    ]
 
 -- | Render the elements of a vector.
 --
@@ -240,7 +225,8 @@ vector :: (G.Vector v a, ToJSON a) =>
        -> Value
 {-# SPECIALIZE vector :: T.Text -> U.Vector Double -> Value #-}
 vector name v = toJSON . map val . G.toList $ v where
-    val i = object [ name .= i ]
+    val i = object [ toKey name .= i ]
+
 
 -- | Render the elements of two vectors.
 vector2 :: (G.Vector v a, G.Vector v b, ToJSON a, ToJSON b) =>
@@ -253,10 +239,19 @@ vector2 :: (G.Vector v a, G.Vector v b, ToJSON a, ToJSON b) =>
                        -> Value #-}
 vector2 name1 name2 v1 v2 = toJSON $ zipWith val (G.toList v1) (G.toList v2) where
     val i j = object
-        [ name1 .= i
-        , name2 .= j
+        [ toKey name1 .= i
+        , toKey name2 .= j
         ]
 
+#if MIN_VERSION_aeson(2,0,0)
+toKey :: T.Text -> Key.Key
+toKey = Key.fromText
+#else
+toKey :: T.Text -> T.Text
+toKey = id
+#endif
+
+
 -- | Attempt to include the contents of a file based on a search path.
 -- Returns 'B.empty' if the search fails or the file could not be read.
 --
